-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- db.lua

function createDB()
	print("createDB")

	-- create system table
	statement = [[
	CREATE TABLE _system
		(   _system_id INTEGER PRIMARY KEY, 
			_system_fullscreen int,
			_system_resx int,
			_system_resy int,
			_system_shadows int,
			_system_vsynch int
		)
	]]
	sql:exe( statement )

	-- add default settings
	statement = [[
	INSERT INTO _system
		( _system_fullscreen, _system_resx, _system_resy, _system_shadows, _system_vsynch )
		VALUES ( 0, 900, 600, 1, 0 )
	]]
	sql:exe( statement )

	-- create classes table
	statement = [[
	CREATE TABLE table_classes
		(   class_id INTEGER PRIMARY KEY, 
			class_name text,
			class_wpn int,
			class_arm int,
			class_stat int,
			class_graphic int,
			class_campaign_id int
		)
	]]
	sql:exe( statement )

	-- create graphics table
	statement = [[
	CREATE TABLE table_graphics
		(   graphic_id INTEGER PRIMARY KEY, 
			graphic_icon text,
			graphic_sprite text,
			graphic_model text,
			graphic_texture text
		)
	]]
	sql:exe( statement )

	-- create items table
	statement = [[
	CREATE TABLE table_items
		(   item_id INTEGER PRIMARY KEY, 
			item_name text,
			item_type int,
			item_class int,
			item_effect int,
			item_stat int,
			item_graphic int,
			item_campaign_id int
		)
	]]
	sql:exe( statement )

	-- create specials table
	statement = [[
	CREATE TABLE table_specials
		(   special_id INTEGER PRIMARY KEY, 
			special_name text,
			special_level int,
			special_class int,
			special_effect int,
			special_stats int
		)
	]]
	sql:exe( statement )

	-- create stats table
	statement = [[
	CREATE TABLE table_stats
		(   stat_id INTEGER PRIMARY KEY, 
			stat_attack int,
			stat_defense int,
			stat_speed int,
			stat_range int,
			stat_damage int,			
			stat_drain int,
			stat_sp int,
			stat_hp int
		)
	]]
	sql:exe( statement )

	-- create high scores table
	statement = [[
	CREATE TABLE table_scores
		(   score_id INTEGER PRIMARY KEY, 
			score_name text,
			score_level int,
			score_depth int,
			score_total int,
			score_campaign_id int
		)
	]]
	sql:exe( statement )	

	-- create encounters table
	statement = [[
	CREATE TABLE table_encounters
		(   encounter_id INTEGER PRIMARY KEY, 
			encounter_mob_name text,
			encounter_group_id int,
			encounter_mob_id int,
			encounter_mobs_min int,
			encounter_mobs_max int,
			encounter_chance int,
			encounter_type int
		)
	]]
	sql:exe( statement )

	-- create groups table
	statement = [[
	CREATE TABLE table_groups
		(   group_id INTEGER PRIMARY KEY, 
			group_name text,
			group_level_min int,
			group_level_max int
		)
	]]
	sql:exe( statement )

	-- create characters table
	statement = [[
	CREATE TABLE table_characters
		(   character_id INTEGER PRIMARY KEY, 
			character_name text,
			character_owner_id int,
			character_class int,
			character_level int,
			character_exp int,
			character_hp int,
			character_sp int,
			character_status int,
			character_wpn int,
			character_arm int,
			character_itm int
		)
	]]
	sql:exe( statement )

	-- create inventory table
	statement = [[
	CREATE TABLE table_inventory
		(   inventory_id INTEGER PRIMARY KEY, 
			inventory_user_id text,
			inventory_item_id int,
			inventory_storage int
		)
	]]
	sql:exe( statement )

	-- create user table
	statement = [[
	CREATE TABLE table_users
		(   user_id INTEGER PRIMARY KEY, 
			user_name text,
			user_gold int
		)
	]]
	sql:exe( statement )
	
	-- create tiles table
	statement = [[
	CREATE TABLE table_tiles
		(   tile_id INTEGER PRIMARY KEY, 
			tile_name text,
			tile_model text,
			tile_texture text,
			tile_type int,
			tile_set int,
			tile_scale int,
			tile_base int
		)
	]]
	sql:exe( statement )
	
	-- create stages table
	statement = [[
	CREATE TABLE table_stages
		(   stage_id INTEGER PRIMARY KEY, 
			stage_name text,
			stage_desc text,
			stage_type text,
			stage_depth int,
			stage_size int,
			stage_level_min int,
			stage_level_max int
		)
	]]
	sql:exe( statement )

	-- add default settings
	statement = [[
	INSERT INTO table_stages
		( stage_name )
		VALUES ( "test")
	]]
	sql:exe( statement )
	
	-- create dialog table
	statement = [[
	CREATE TABLE table_dialog
		(   dialog_id INTEGER PRIMARY KEY, 
			dialog_stage int,
			dialog_depth int,
			dialog_target int,
			dialog_type int,
			dialog_char_index int,
			dialog_text text,
			dialog_image text,			
			dialog_name text,
			dialog_icon text,
			dialog_sound text,
		)
	]]
	sql:exe( statement )
	
	-- create effects table
	statement = [[
	CREATE TABLE table_effects
		(   effect_id INTEGER PRIMARY KEY, 
			effect_name text,
			effect_type int,
			effect_duration int,
			effect_magnitude int,
			effect_break int,
			effect_stack int,
			effect_stats int,			
			effect_icon text,
			effect_animation text
		)
	]]
	sql:exe( statement )

	-- add default settings
	statement = [[
	INSERT INTO table_effects
		( effect_name )
		VALUES ( "test")
	]]
	sql:exe( statement )
end

-- eof