-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- base.lua

base = { ID = 0 }

-- creation
function base:new (o)
	-- print "[base] new base"
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	
	width = 128
	height = 20
	x = screen_width - width - 40
	y = 100

	-- unit window and reserve selection list.
	unit_window = sim:addWindow( 300, xcore.x2drect(x, y, x+width, y+256+30), "Deployment List")
	unit_window:setTitleBarVisible( false )
	image_background_base = sim:addImage( 0, texture_background_128x256, xcore.x2dpoint(0, 0),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
	image_background_base:setParent( unit_window )

	width = 118	
	x = 5
	y = 10
	listbox_unit_list =  sim:addListbox( 302, true, xcore.x2drect(x, y, x+width, y+240 ) )
	listbox_unit_list:setParent( unit_window )

	x = 0
	y = 256
	width = 128	
	button_deploy = sim:addButton( 303, xcore.x2drect( x, y, x+width, y+28 ), "Deploy" )
	button_deploy:setParent( unit_window )
	button_deploy:setImage( button_texture )
	button_deploy:setPressedImage( button_pressed_texture )

	sim:bindGUI ( "button_deploy_pressed", 303, xcore.XGUI_BUTTON_CLICKED )

	self:hide()
	return o
end

function base:hide()
	-- print "[base] hide"
	unit_window:setVisible( false )
end

function base:show()
	-- print "[base] show"
	self:update()
	unit_window:setVisible( true )
	-- print "[base] /show"
end

function base:destroy()
	-- print "[base] destroy"
	listbox_unit_list:destroy()
	unit_window:destroy()
end

function base:deploy()
	-- print( "[base]deploy unit" )
	-- first get selected
	local tableindex = listbox_unit_list:getSelected()	

	if tableindex == nil then return nil end
	if tableindex == -1 then return nil end
	
	unit_name = listbox_unit_list:getItem( tableindex )
	-- print("[base] unit name : " .. unit_name )
	local unit_index = getMobIndexByName( unit_name )
	
	self:deployMobByIndex( unit_index )

end

function base:deployMobByIndex( i )
	-- move unit to base panel
	if allied_activations <= 0 or allies_in_progress ~= 0 then
		-- print( "[base] error; no activations" )
		game_history:addItem( "A unit on map must end it's activation before a new unit may deploy." )
		return nil
	end

	if i ~= nil then
		for x = 1, map_size do
			for y = 1, map_size do
				if game_map.tiles[ x ][ y ].tile_type == TILE_TYPE_START then
					game_map.mobs[ i ]:setGrid( x, y )
					game_map.mobs[ i ]:setDest( x, y )
					game_map.mobs[ i ].state = STATE_READY
					unit_name = game_map.mobs[ i ]:getName()

				end
			end
		end
	else
		print("[base] nil index")
	end
	game_history:addItem( unit_name .. " is now active!" )
	start_indicator:setVisible( false )
	exit_indicator:setVisible( false )
	
	-- marked as selected
	selected = i
	-- show action menu / hide base panel
	self:hide()
	menu_actions:show()
	menu_info:show()
end

function base:update()
	-- print "[base] update"
	listbox_unit_list:clear()
	for n = 0, table.getn( game_map.mobs )-1 do
		-- print( "[base] n: " .. n+1 )
		-- print( "[base] mob : " .. game_map.mobs[ n+1 ].name )
		-- print( "[base] state: " .. game_map.mobs[ n+1 ].active )
		if game_map.mobs[ n+1 ].owner_id == 1 and game_map.mobs[ n+1 ].state == STATE_RESERVE then
			listbox_unit_list:addItem ( game_map.mobs[ n+1 ].name )
		end
	end
	-- print "[base] /update"
end

function getMobIndexByName( t )
	-- print "[manager] getMobIndexByName"
	local i = table.getn( game_map.mobs )
	-- print ( "[manager] objects.count = " .. i )
	for n = 1, i do
		-- print ("[manager] match against name : " .. t )
		if game_map.mobs[ n ].name == t and game_map.mobs[ n ].state == STATE_RESERVE then
			-- print ( "[manager] return " .. n )
			return n
		end
	end
	-- print "[manager] return null"
	return nil
end

function getMobIndexByGrid( x, y )
	-- print "[manager] getMobIndexByName"
	local i = table.getn( game_map.mobs )
	-- print ( "[manager] objects.count = " .. i )
	for n = 1, i do
		-- print ("[manager] match against name : " .. t )
		if game_map.mobs[ n ].grid.X == x and  game_map.mobs[ n ].grid.Y == y then
			-- print ( "[manager] return " .. n )
			return n
		end
	end
	-- print "[manager] return null"
	return nil
end

-- end of file