-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- dialog.lua

dialog = { ID = 0 }

-- creation
function dialog:new (o)
	-- print "[dialog] new dialog"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	self.ID       = 0

	-- this is the dialog menu. it is the conversation system
	-- and allows players to view the lines of text dialog,
	-- supporting graphics, and audio associated with a level.

	
	-- NPC dialog box
	x = screen_width/2-320
	y = screen_height/2-180
	
	-- frame
	texture_dialog_background = sim:getTexture( path .. "resources/interface/dialog_box.png" )
	image_background_npc_dialog = sim:addImage( 0, texture_dialog_background, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )

	-- icon
	x = screen_width/2-300
	y = screen_height/2-142
	texture_dialog_icon_npc = sim:getTexture( path .. "resources/interface/icon_.png" )
	image_dialog_icon_npc = sim:addImage( 0, texture_dialog_icon_npc, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )	

	-- name (X)
	x = screen_width/2-310
	y = screen_height/2-178
	text_npc_name = sim:addText( 0, "NPC Name.", x, y, x+width, y+height)
	
	-- dialog (X)
	x = screen_width/2-218
	y = screen_height/2-162
	x = x + 8
	y = y + 8
	width = 560
	height = 80
	text_npc_dialog = sim:addText( 0, "NPC Dialog.", x, y, x+width, y+height)


	-- PC dialog box
	x = screen_width/2-320
	y = screen_height/2+80

	-- frame
	btn_w = 560
	btn_h = 26	
	image_background_player_dialog = sim:addImage( 0, texture_dialog_background, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )
		
	-- icon
	x = screen_width/2-300
	y = screen_height/2+112

	texture_dialog_icon_pc = sim:getTexture( path .. "resources/interface/icon_.png" )
	image_dialog_icon_pc = sim:addImage( 0, texture_dialog_icon_npc, xcore.x2dpoint(x, y),
		xcore.x2drect(0,0,64,64), xcore.xcolour(255,255,255,255) )

	-- name
	x = screen_width/2-310
	y = screen_height/2+82
	text_pc_name = sim:addText( 0, "PC Name.", x, y, x+width, y+height)

	-- options
	x = screen_width/2-220
	y = screen_height/2+100
	x = x + 8
	y = y + 8
	dialog_option_1 = sim:addButton( 2501, xcore.x2drect( x, y, x+btn_w, y+btn_h ), "1) opt1" )
	y = y + 26
	dialog_option_2 = sim:addButton( 2502, xcore.x2drect( x, y, x+btn_w, y+btn_h ), "2) opt2" )	
	y = y + 26
	dialog_option_3 = sim:addButton( 2503, xcore.x2drect( x, y, x+btn_w, y+btn_h ), "3) opt3" )	
	


	sim:bindGUI ( "dialog_option_1_pressed", 2501, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "dialog_option_2_pressed", 2502, xcore.XGUI_BUTTON_CLICKED )
	sim:bindGUI ( "dialog_option_3_pressed", 2503, xcore.XGUI_BUTTON_CLICKED )

	-- player dialog box should be split into 3 reply area hotspot/buttons?
	
	self.step = 1
	self.opt1_target = 0
	self.opt2_target = 0
	self.opt3_target = 0
	
	self:hide()
	return o
end

function dialog:show()
	-- print("[dialog] show")
	text_npc_dialog:setVisible( true )
	image_background_npc_dialog:setVisible( true )	
	image_background_player_dialog:setVisible( true )
	-- dialog_option_1:setVisible( true )
	-- dialog_option_2:setVisible( true )
	-- dialog_option_3:setVisible( true )
	text_npc_name:setVisible( true )	
	text_pc_name:setVisible( true )	
	image_dialog_icon_npc:setVisible( true )	
	image_dialog_icon_pc:setVisible( true )	
	
	self.visible = 1
end

function dialog:hide()
	-- print("[dialog] hide")
	text_npc_dialog:setVisible( false )
	image_background_npc_dialog:setVisible( false )	
	image_background_player_dialog:setVisible( false )
	dialog_option_1:setVisible( false )
	dialog_option_2:setVisible( false )
	dialog_option_3:setVisible( false )

	text_npc_name:setVisible( false )	
	text_pc_name:setVisible( false )	
	image_dialog_icon_npc:setVisible( false )	
	image_dialog_icon_pc:setVisible( false )	
	
	self.visible = 0
end

function dialog:input()
	if sim:isGUI( "dialog_option_1_pressed" ) then
		if self.opt1_target ~= 0 then
			self.step = self.opt1_target
			self:update()
		end
	end
	if sim:isGUI( "dialog_option_2_pressed" ) then
		if self.opt2_target ~= 0 then
			self.step = self.opt2_target
			self:update()
		end
	end
	if sim:isGUI( "dialog_option_3_pressed" ) then
		if self.opt3_target ~= 0 then
			self.step = self.opt3_target
			self:update()
		end
	end	
end

function dialog:load( stage, depth )
	-- print("[dialog] load")
	self.dialogTree = {}
	self.dialog_count = 0
	self.step = 1

	-- load all dialog options for this stage
	query = "SELECT dialog_type, dialog_target, dialog_name, dialog_icon, dialog_order, dialog_text, dialog_char_index FROM table_dialog WHERE dialog_stage = " ..  stage .. " AND dialog_depth = " .. depth .." ORDER BY dialog_depth ASC, dialog_order ASC, dialog_type ASC"
	-- print( query )
	local q = sql:exe( query )
	if q ~= nil then
		-- print ("returned " .. q:getRowCount() .. " rows ")		
		for i = 0, q:getRowCount()-1 do
			local n = i*q:getHeadCount()
			-- print("\ntype : "    .. q:getData( n+0 ) )
			-- print(" target : "    .. q:getData( n+1 ) )
			-- print(" name : "   .. q:getData( n+2 ) )
			-- print(" icon : " .. q:getData( n+3 ) )
			-- print(" order : "   .. q:getData( n+4 ) )
			-- print(" text : "   .. q:getData( n+5 ) )
			-- print(" char_index : "   .. q:getData( n+6 ) )

			self.dialogTree[i] = {}
			self.dialogTree[i]["type"]       = tonumber( q:getData( n+0 ) )
			self.dialogTree[i]["target"]     = tonumber( q:getData( n+1 ) )
			self.dialogTree[i]["name"]       = q:getData( n+2 )
			self.dialogTree[i]["icon"]       =  q:getData( n+3 )
			self.dialogTree[i]["order"]      = tonumber( q:getData( n+4 ) )
			self.dialogTree[i]["text"]       = q:getData( n+5 )
			self.dialogTree[i]["char_index"] = tonumber( q:getData( n+6 ) )
			
			self.dialog_count = self.dialog_count + 1
		end
	else
		-- print("[Dialog] Error : No dialog found")
	end
	self:update()	
end

function dialog:update()
	-- print("[dialog] update")
	-- print("[dialog] step: " .. self.step )
	-- print("[dialog] dialog_count: " .. self.dialog_count )

	local tmp = 0
	local count = 0
	self.opt1_target = 0
	self.opt2_target = 0
	self.opt3_target = 0

	dialog_option_1:setVisible( false )
	dialog_option_2:setVisible( false )
	dialog_option_3:setVisible( false )

	dialog_option_1:setText("")
	dialog_option_2:setText("")
	dialog_option_3:setText("")
	text_npc_dialog:setText("")

	if self.dialog_count > 0 then
		for i = 0, table.getn( self.dialogTree ) do
			-- print("self.dialogTree[i][order]: " .. self.dialogTree[i]["order"] )
			if self.dialogTree[i]["order"] == self.step then
				count = count + 1
				if self.dialogTree[i]["type"] == 0 then -- statement
					-- print("[dialog] statement")
					text_npc_dialog:setText( self.dialogTree[i]["text"] )
					self.opt1_target = self.dialogTree[i]["target"]
					if self.dialogTree[i]["target"] == -1 then
						dialog_option_1:setText( "Done" )
					else
						dialog_option_1:setText( "More" )
					end
					dialog_option_1:setVisible(true)

					if self.dialogTree[i]["char_index"] ~= nil then
						-- print("")
						image_dialog_icon_npc:setImage( sim:getTexture( path .. "resources/sprites/icon_" .. game_map.mobs[ self.dialogTree[i]["char_index"] ].graphic .. ".png" ) )
						if game_map.mobs[ self.dialogTree[i]["char_index"] ].name == "none" or game_map.mobs[ self.dialogTree[i]["char_index"] ].name == game_map.mobs[ self.dialogTree[i]["char_index"] ].class_name then
							text_npc_name:setText( game_map.mobs[ self.dialogTree[i]["char_index"] ].class_name )
						else
							text_npc_name:setText(  game_map.mobs[ self.dialogTree[i]["char_index"] ].name  .. " the " .. game_map.mobs[ self.dialogTree[i]["char_index"] ].class_name )
						end
					else
						if self.dialogTree[i]["icon"] ~= nil then
							image_dialog_icon_npc:setImage( sim:getTexture( path .. "resources/sprites/icon_" .. self.dialogTree[i]["icon"] .. ".png" ) )	
						end
						if self.dialogTree[i]["name"] ~= nil then
							text_npc_name:setText( self.dialogTree[i]["name"] )
						end
						-- print("[dialog] no icon data")
					end
				elseif self.dialogTree[i]["type"] == 1 then -- question
					-- print("[dialog] question")
					text_npc_dialog:setText( self.dialogTree[i]["text"] )

					if self.dialogTree[i]["char_index"] ~= nil then
						image_dialog_icon_npc:setImage( sim:getTexture( path .. "resources/sprites/icon_" .. game_map.mobs[ self.dialogTree[i]["char_index"] ].graphic .. ".png" ) )
						if game_map.mobs[ self.dialogTree[i]["char_index"] ].name == "none" or game_map.mobs[ self.dialogTree[i]["char_index"] ].name == game_map.mobs[ self.dialogTree[i]["char_index"] ].class_name then
							text_npc_name:setText( game_map.mobs[ self.dialogTree[i]["char_index"] ].class_name )
						else
							text_npc_name:setText(  game_map.mobs[ self.dialogTree[i]["char_index"] ].name  .. " the " .. game_map.mobs[ self.dialogTree[i]["char_index"] ].class_name )
						end
					else
						if self.dialogTree[i]["icon"] ~= nil then
							image_dialog_icon_npc:setImage( sim:getTexture( path .. "resources/sprites/icon_" .. self.dialogTree[i]["icon"] .. ".png" ) )	
						end
						if self.dialogTree[i]["name"] ~= nil then
							text_npc_name:setText( self.dialogTree[i]["name"] )
						end
						-- print("[dialog] no icon data")
					end

				elseif self.dialogTree[i]["type"] == 2 then -- answer
					-- print("[dialog] answer")
					tmp = tmp + 1
					if tmp == 1 then
						dialog_option_1:setText( self.dialogTree[i]["text"] )
						self.opt1_target = self.dialogTree[i]["target"]
						dialog_option_1:setVisible(true)
					elseif tmp == 2 then
						dialog_option_2:setText( self.dialogTree[i]["text"] )
						self.opt2_target = self.dialogTree[i]["target"]
						dialog_option_2:setVisible(true)
					elseif tmp == 3 then
						dialog_option_3:setText( self.dialogTree[i]["text"] )
						self.opt3_target = self.dialogTree[i]["target"]
						dialog_option_3:setVisible(true)
					end
				end
			end
		end
	else
		self.step = -1
	end

	image_dialog_icon_pc:setImage( sim:getTexture( path .. "resources/sprites/icon_" .. game_map.mobs[ 1 ].graphic .. ".png" ) )
	
	if game_map.mobs[ 1 ].name == "none" or game_map.mobs[ 1 ].name == game_map.mobs[ 1 ].class_name then
		text_pc_name:setText( game_map.mobs[ 1 ].class_name )
	else
		text_pc_name:setText(  game_map.mobs[ 1 ].name  .. " the " .. game_map.mobs[ 1 ].class_name )
	end

	if count == 0 then
		-- no dialog defined this step, dont show dialog window
		-- print("no dialog this step")
		self.step = -1
	end

	if self.step == -1 then
		-- print("[dialog] step == -1")	
		self:hide()	
		if substate == SUBSTATE_INTRO then
			-- print("[dialog] finish start dialog")
			substate = SUBSTATE_INPUT
		elseif substate == SUBSTATE_EXIT then
			-- print("[dialog] finish end dialog")
			substate = SUBSTATE_OVER
		end
	end
end

function dialog:changeState( n )
	-- print("[dialog] changeState")
end



-- eof