-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- mob.lua

mob = { ID = 0 }

STATE_RESERVE   = 0
STATE_READY     = 1
STATE_ACTIVE    = 2
STATE_DONE      = 3
STATE_EXITED    = 4
STATE_FLED      = 5
STATE_DOWN      = 6

ANIMATION_TYPE_IDLE    = 0
ANIMATION_TYPE_WALK    = 1
ANIMATION_TYPE_ATTACK  = 2
ANIMATION_TYPE_SPECIAL = 3
ANIMATION_TYPE_DIE     = 4

FACING_TYPE_NORTH      = 1 -- front
FACING_TYPE_WEST       = 2 -- left
FACING_TYPE_SOUTH      = 3 -- back
FACING_TYPE_EAST       = 4 -- right

-- creation
function mob:new (o)
	-- print "[mob] new mob"
	o = o or {}
	setmetatable(o, self)
	self.__index = self

	self.type       = 0
	self.mob_id     = 0
	self.parent_id  = 0
	self.owner_id   = 0
	self.ani_freq   = 25
 
	-- AI
	self.target_id       = 0
	self.range_to_target = 999

	self.name     = "none"
	self.class_name     = "none"
	self.status   = 0
	self.state    = 0 -- 0 = reserve, 1 = ready, 2 = moved, 3 = attacked, 4 = done, 5 exited, -1 fled, -2 = downed, -3 killed 

	o.grid        = xcore.x2dpoint( 0, 0 )
	o.dest        = xcore.x2dpoint( 0, 0 )	
	o.position    = xcore.x3dpoint( 0, 0, 0 )

	self.step      = 0 -- used for move animation
	self.has_moved = 0

	-- stats (current)
	self.attack  = 2
	self.defense = 2
	self.move    = 2
	self.range   = 1
	self.damage  = 1
	self.drain   = 1
	self.hp      = 1
	self.sp      = 2

	-- stats (base)
	self.max_attack  = 2
	self.max_defense = 2
	self.max_move    = 2
	self.max_range   = 1
	self.max_damage  = 1
	self.max_drain   = 1
	self.max_hp      = 1
	self.max_sp      = 2

	self.class        = 1
	self.wpn_skill    = 0
	self.arm_skill    = 0

	self.wpn_equipped_id = 0
	self.arm_equipped_id = 0
	self.acc_equipped_id = 0

	self.graphic = "unknown"
	
	self.ani_frame  = 1
	self.ani_time   = 0
	self.ani_face   = 4
	self.ani_set    = 1
	self.loop       = 1
	self.ani_len    = 9
	self.mob_face   = 4
	self.wait       = 0

	self.level = 1
	self.exp   = 0

	o.special_abilities = {}
	o.status_effects = {}
	-- self.effect = "None" -- burn, blead, poison, slow, immobile, sleep, stun, confused

	-- gear
	-- self.wep = {}
	-- self.arm = {}
	-- self.acc = {}

	return o
end

function mob:getName()
	-- print("[mob] getName")
	if self.name == "none" then
		return self.class_name
	else
		return self.name 
	end
end

function mob:getTotalAttack()
	-- print("[mob] getTotalAttack")
	local total = self.max_attack
	if self.wpn ~= nil then total = total + self.wpn.stat_attack end
	if self.arm ~= nil then total = total + self.arm.stat_attack end
	if self.acc ~= nil then total = total + self.acc.stat_attack end
	
	-- loop over effects checking for stat alters
	if table.getn( self.status_effects ) > 0 then
		-- print("[mob] has status effects")
		for n = 1, table.getn( self.status_effects ) do
			-- print("type : " .. self.status_effects[ n ].effect_type )
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_attack
			end
		end
	end
	return total
end

function mob:getTotalDefense()
	local total = self.max_defense
	if self.wpn ~= nil then total = total + self.wpn.stat_defense end
	if self.arm ~= nil then total = total + self.arm.stat_defense end
	if self.acc ~= nil then total = total + self.acc.stat_defense end
	-- loop over effects checking for stat alters
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_defense
			end
		end
	end
	return total
end

function mob:getTotalMove()
	local total = self.max_move
	if self.wpn ~= nil then total = total + self.wpn.stat_move end
	if self.arm ~= nil then total = total + self.arm.stat_move end
	if self.acc ~= nil then total = total + self.acc.stat_move end
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_move
			end
		end
	end	
	return total
end

function mob:getTotalRange()
	local total = self.max_range
	if self.wpn ~= nil then total = total + self.wpn.stat_range end
	if self.arm ~= nil then total = total + self.arm.stat_range end
	if self.acc ~= nil then total = total + self.acc.stat_range end
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_range
			end
		end
	end		
	return total
end

function mob:getTotalDamage()
	local total = self.max_damage
	if self.wpn ~= nil then total = total + self.wpn.stat_damage end
	if self.arm ~= nil then total = total + self.arm.stat_damage end
	if self.acc ~= nil then total = total + self.acc.stat_damage end
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_damage
			end
		end
	end		
	return total
end

function mob:getTotalDrain()
	local total = self.max_drain
	if self.wpn ~= nil then total = total + self.wpn.stat_drain end
	if self.arm ~= nil then total = total + self.arm.stat_drain end
	if self.acc ~= nil then total = total + self.acc.stat_drain end
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_drain
			end
		end
	end		
	return total
end

function mob:getTotalHP()
	local total = self.max_hp * self.level
	if self.wpn ~= nil then total = total + self.wpn.stat_hp end
	if self.arm ~= nil then total = total + self.arm.stat_hp end
	if self.acc ~= nil then total = total + self.acc.stat_hp end
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_hp
			end
		end
	end
	return total
end

function mob:getTotalSP()
	local total = self.max_sp * self.level
	if self.wpn ~= nil then total = total + self.wpn.stat_sp end
	if self.arm ~= nil then total = total + self.arm.stat_sp end
	if self.acc ~= nil then total = total + self.acc.stat_sp end
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_BUFF or self.status_effects[ n ].effect_type == EFFECT_TYPE_DEBUFF then
				total = total + self.status_effects[ n ].effect_sp
			end
		end
	end
	return total
end

function mob:destroy()
	-- print("[mob:destroy]")
	if self.mesh ~= nil then self.mesh:destroy() end
	if self.billboard ~= nil then self.billboard:destroy() end
	if self.sprites ~= nil then
		-- self.billboard:setPosition( xcore.x3dpoint( self.grid.X* tile_size, 4, self.grid.Y* tile_size ) )
		self.sprites[1]:destroy()
		self.sprites[2]:destroy()
		self.sprites[3]:destroy()
		self.sprites[4]:destroy()
	end
	if self.action_icon ~= nil then
		self.action_icon:destroy()
	end

	n = table.getn( self.status_effects )
	for i = n, 1, -1 do
		table.remove ( self.status_effects, i )
	end
	n = table.getn( self.special_abilities )
	for i = n, 1, -1 do
		table.remove ( self.special_abilities, i )
	end
	
end

function mob:save()
end

function mob:setDest( x, y )
	self.dest.X = x
	self.dest.Y = y
end

function mob:setGrid( x, y )
	-- print("[mob] setGrid")
	self.grid.X = x
	self.grid.Y = y

	self:setPosition( xcore.x3dpoint( self.grid.X* tile_size, 3.5, self.grid.Y* tile_size ) )
end

function mob:setPosition( p )
	-- print("[mob] setPosition")
	self.position = p

	if game_map.tiles ~= nil then
		-- print("[mob] grid :" .. self.grid.X .. ", " .. self.grid.Y )
		-- print("[mob] pos :" .. self.position.X .. ", " .. self.position.Y .. ", " .. self.position.Z )
		
		if self.grid.X >= 1 and self.grid.X <= map_size and self.grid.Y >= 1 and self.grid.Y <= map_size then
			if game_map.tiles[ self.grid.X ][ self.grid.Y ].tile_height == 1 then
				-- print("adjust tile height")
				self.position.Y = 10
			elseif game_map.tiles[ self.grid.X ][ self.grid.Y ].tile_height == -1 then
				-- print("adjust tile height")
				self.position.Y = 0
			else
				self.position.Y = 4.5
			end
		end
	end

	if self.mesh ~= nil then
		self.mesh:setPosition( self.position)
	end
	
	if self.sprites ~= nil then
		self.sprites[1]:setPosition( self.position )
		self.sprites[2]:setPosition( self.position )
		self.sprites[3]:setPosition( self.position )
		self.sprites[4]:setPosition( self.position )
	end

	if self.billboard ~= nil then
		self.billboard:setPosition( self.position )
	end
	if self.action_icon ~= nil then
		self.action_icon:setPosition( xcore.x3dpoint( self.position.X-2, self.position.Y+6, self.position.Z  ) )
	end
end

function mob:loadSpecials()
	-- print ("[mob] loadSpecials")
	-- que class data from database

	statement = "SELECT special_name, special_effect, special_level, stat_attack, stat_defense, stat_speed, stat_range, stat_damage, stat_drain, stat_sp, stat_hp,  special_stats, special_radius, special_type, special_graphic "
	statement = statement .. " FROM table_specials "
	statement = statement .. " LEFT JOIN table_stats on stat_id = special_stats "
	statement = statement .. " WHERE special_class = " .. self.class
	statement = statement .. " ORDER BY special_level ASC"
	-- print( statement )
	local q = sql:exe( statement )
	if q == nil then
		-- print(" [mob] no specials found")
	else
		for i = 0, q:getRowCount()-1 do
			local n = i*q:getHeadCount()
			-- print("[mob] special found")
			p = table.getn( self.special_abilities ) + 1
			self.special_abilities[ p ] = ability:new()
			if q:getData( n+0 ) ~= nil then self.special_abilities[ p ].ability_name = q:getData( n+0 ) end -- todo: fixme?
			if q:getData( n+1 ) ~= nil then self.special_abilities[ p ].ability_effect = tonumber( q:getData( n+1 ) ) end
			if q:getData( n+2 ) ~= nil then self.special_abilities[ p ].ability_level = tonumber( q:getData( n+2 ) ) end
			if q:getData( n+3 ) ~= nil then self.special_abilities[ p ].stat_attack  = tonumber( q:getData( n+3 ) ) end
			if q:getData( n+4 ) ~= nil then self.special_abilities[ p ].stat_defense = tonumber( q:getData( n+4 ) ) end
			if q:getData( n+5 ) ~= nil then self.special_abilities[ p ].stat_move    = tonumber( q:getData( n+5 ) ) end
			if q:getData( n+6 ) ~= nil then self.special_abilities[ p ].stat_range   = tonumber( q:getData( n+6 ) ) end
			if q:getData( n+7 ) ~= nil then self.special_abilities[ p ].stat_damage  = tonumber( q:getData( n+7 ) ) end
			if q:getData( n+8 ) ~= nil then self.special_abilities[ p ].stat_drain   = tonumber( q:getData( n+8 ) ) end
			if q:getData( n+9 ) ~= nil then self.special_abilities[ p ].stat_sp      = tonumber( q:getData( n+9 ) ) end
			if q:getData( n+10 ) ~= nil then self.special_abilities[ p ].stat_hp      = tonumber( q:getData( n+10 ) ) end
			if q:getData( n+12 ) ~= nil then self.special_abilities[ p ].ability_radius      = tonumber( q:getData( n+12 ) ) end
			if q:getData( n+13 ) ~= nil then self.special_abilities[ p ].ability_type      = tonumber( q:getData( n+13 ) ) end
			if q:getData( n+14 ) ~= nil then self.special_abilities[ p ].ability_graphic      =  q:getData( n+14 ) end
			-- print("[mob]ability name : " .. self.special_abilities[ p ].ability_name )
		end
		sql:removeResult(q)		
	end

	
	-- attach effects to specials?
	for p = 1, table.getn( self.special_abilities ) do
		-- print("[mob] ability:" .. self.special_abilities[ p ].ability_name )
		-- print("[mob] effect id:" .. self.special_abilities[ p ].ability_effect )
		if self.special_abilities[ p ].ability_effect > 0 then
			statement = "SELECT effect_name, effect_type, effect_duration, effect_magnitude, effect_break, effect_stack, effect_icon, effect_animation, stat_attack, stat_defense, stat_speed, stat_range, stat_damage, stat_drain, stat_sp, stat_hp "
			statement = statement .. " FROM table_effects "
			statement = statement .. " LEFT JOIN table_stats on stat_id = effect_stats "
			statement = statement .. " WHERE effect_id = " ..  self.special_abilities[ p ].ability_effect
			-- print( statement )
			local q = sql:exe( statement )
			if q ~= nil then
				for i = 0, q:getRowCount()-1 do
				local n = i*q:getHeadCount()
			
					if q:getData( n+0 ) ~= nil then
						self.special_abilities[ p ].effect_name      = q:getData( n+0 )
						-- print("[mob] effect name:" .. self.special_abilities[ p ].effect_name )
					end -- todo: fixme?
					if q:getData( n+1 ) ~= nil then self.special_abilities[ p ].effect_type      = tonumber( q:getData( n+1 ) ) end
					if q:getData( n+2 ) ~= nil then self.special_abilities[ p ].effect_duration  = tonumber( q:getData( n+2 ) ) end
					if q:getData( n+3 ) ~= nil then self.special_abilities[ p ].effect_magnitude = tonumber( q:getData( n+3 ) ) end
					if q:getData( n+4 ) ~= nil then self.special_abilities[ p ].effect_break     = tonumber( q:getData( n+4 ) ) end
					if q:getData( n+5 ) ~= nil then self.special_abilities[ p ].effect_stack     = tonumber( q:getData( n+5 ) ) end
					if q:getData( n+6 ) ~= nil then self.special_abilities[ p ].effect_icon      = tonumber( q:getData( n+6 ) ) end
					if q:getData( n+7 ) ~= nil then self.special_abilities[ p ].effect_animation = tonumber( q:getData( n+7 ) ) end

					if q:getData( n+8 ) ~= nil then self.special_abilities[ p ].effect_attack  = tonumber( q:getData( n+8 ) ) end
					if q:getData( n+9 ) ~= nil then self.special_abilities[ p ].effect_defense = tonumber( q:getData( n+9 ) ) end
					if q:getData( n+10 ) ~= nil then self.special_abilities[ p ].effect_move    = tonumber( q:getData( n+10 ) ) end
					if q:getData( n+11 ) ~= nil then self.special_abilities[ p ].effect_range   = tonumber( q:getData( n+11 ) ) end
					if q:getData( n+12 ) ~= nil then self.special_abilities[ p ].effect_damage  = tonumber( q:getData( n+12 ) ) end
					if q:getData( n+13 ) ~= nil then self.special_abilities[ p ].effect_drain   = tonumber( q:getData( n+13 ) ) end
					if q:getData( n+14 ) ~= nil then self.special_abilities[ p ].effect_sp      = tonumber( q:getData( n+14 ) ) end
					if q:getData( n+15 ) ~= nil then self.special_abilities[ p ].effect_hp      = tonumber( q:getData( n+15 ) ) end
				end
			end
			sql:removeResult(q)			
		end
	end
end

function mob:setClass( n )
	-- print ("[mob] setClass")
	-- print("class : " .. n)
	-- que class data from database
	self.class = n
	statement = "SELECT class_name, class_wpn, class_arm, stat_attack, stat_defense, stat_speed, stat_range, stat_damage, stat_drain, stat_sp, stat_hp,  class_stat, class_type, class_graphic "
	statement = statement .. " FROM table_classes "
	statement = statement .. " LEFT JOIN table_stats on stat_id = class_stat "
	statement = statement .. " WHERE class_id = " .. n
	-- print( statement )
	local q = sql:exe( statement )
	if q == nil then
		-- print(" row not found")
		self.graphic = "unknown"
	else
		-- print(" row found")
	
		if q:getData( 0 ) ~= nil then self.class_name = q:getData( 0 ) end -- todo: fixme?
		if q:getData( 1 ) ~= nil then self.wpn_skill = tonumber( q:getData( 1 ) ) end
		if q:getData( 2 ) ~= nil then self.arm_skill = tonumber( q:getData( 2 ) ) end
		if q:getData( 3 ) ~= nil then self.max_attack  = tonumber( q:getData( 3 ) ) end
		if q:getData( 4 ) ~= nil then self.max_defense = tonumber( q:getData( 4 ) ) end
		if q:getData( 5 ) ~= nil then self.max_move    = tonumber( q:getData( 5 ) ) end
		if q:getData( 6 ) ~= nil then self.max_range   = tonumber( q:getData( 6 ) ) end
		if q:getData( 7 ) ~= nil then self.max_damage  = tonumber( q:getData( 7 ) ) end
		if q:getData( 8 ) ~= nil then self.max_drain   = tonumber( q:getData( 8 ) ) end
		if q:getData( 9 ) ~= nil then self.max_sp      = tonumber( q:getData( 9 ) ) end
		if q:getData( 10 ) ~= nil then self.max_hp      = tonumber( q:getData( 10 ) ) end
		if q:getData( 12 ) ~= nil then self.type      = tonumber( q:getData( 12 ) ) end		
		if q:getData( 13 ) ~= nil then self.graphic = q:getData( 13 ) end -- todo: fixme?		
		sql:removeResult(q)
	end
	self:loadGraphic( self.graphic )
	self:loadSpecials()	
	self:reset()
end

function mob:reset()
	-- print ("[mob] resetStats")
	-- stats (current)
	self.level = self:getLevel()
	self.attack  = self.max_attack
	self.defense = self.max_defense
	self.move    = self.max_move
	self.range   = self.max_range
	self.damage  = self.max_damage
	self.drain   = self.max_drain
	self.hp      = self:getTotalHP()
	self.sp      = self:getTotalSP()
end

function mob:loadGraphic( n )
	-- print ("[mob] setGraphic")
	-- print ("[mob] self.mob_id : " .. self.mob_id )
	-- load graphics;
	-- sprite, or mesh and texture
	self.sprites = {}
	self.sprites[1] = sim:addSprite ( 0, path .. "resources/sprites/".. n .."_move.png", path .. "resources/sprites/null.png",
			path .. "resources/sprites/null.png", path .. "resources/sprites/null.png"  )
	self.sprites[1]:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
	self.sprites[1]:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
	self.sprites[1]:setSize( xcore.x2dpoint( 10, 10 ) )
	self.sprites[1]:setVisible(false)

	self.sprites[2] = sim:addSprite ( 0, path .. "resources/sprites/".. n .."_attack.png", path .. "resources/sprites/null.png", path .. "resources/sprites/null.png", path .. "resources/sprites/null.png"  )
	self.sprites[2]:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
	self.sprites[2]:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
	self.sprites[2]:setSize( xcore.x2dpoint( 10, 10 ) )
	self.sprites[2]:setVisible(false)

	self.sprites[3] = sim:addSprite ( 0, path .. "resources/sprites/".. n .."_cast.png", path .. "resources/sprites/null.png", path .. "resources/sprites/null.png", path .. "resources/sprites/null.png"  )
	self.sprites[3]:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
	self.sprites[3]:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
	self.sprites[3]:setSize( xcore.x2dpoint( 10, 10 ) )
	self.sprites[3]:setVisible(false)

	self.sprites[4] = sim:addSprite ( 0, path .. "resources/sprites/".. n .."_hurt.png", path .. "resources/sprites/null.png", path .. "resources/sprites/null.png", path .. "resources/sprites/null.png"  )
	self.sprites[4]:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
	self.sprites[4]:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
	self.sprites[4]:setSize( xcore.x2dpoint( 10, 10 ) )
	self.sprites[4]:setVisible(false)	

	self:setAnimation( ANIMATION_TYPE_IDLE )
	
	
	-- set action icon
	-- set hp bar
	-- set SP bar
	if self.action_icon == nil then
		self.action_icon = sim:addBillboard( -1, xcore.x2dpoint( 2, 2 ) )
		self.action_icon:addTexture( path .. "resources/interface/icon_ready.png", 0 )
		self.action_icon:setMaterialFlag( xcore.XMF_LIGHTING, false )
		self.action_icon:setMaterialFlag( xcore.XMF_ANISOTROPIC_FILTER, true )
		self.action_icon:setMaterialType( xcore.XMT_TRANSPARENT_ALPHA_CHANNEL )
	end
end

function mob:setState( i )
	self.state = i
	if self.action_icon ~= nil then
		if i == STATE_READY then
			self.action_icon:setVisible( true )
			self.action_icon:addTexture( path .. "resources/interface/icon_ready.png", 0 )
		elseif  i == STATE_ACTIVE then
			self.action_icon:setVisible( true )
			self.action_icon:addTexture( path .. "resources/interface/icon_active.png", 0 )
		elseif i == STATE_DONE then
			self.action_icon:setVisible( true )
			self.action_icon:addTexture( path .. "resources/interface/icon_done.png", 0 )
		else
			-- hide icon?
			self.action_icon:setVisible( false )
		end
	end
end

function mob:setTexture( n, i )
	-- print ("[mob] setGraphic")
	-- print ("[mob] self.mob_id : " .. self.mob_id )

	self.sprites[1]:setTexture ( path .. "resources/sprites/".. n .."_move.png", i )
	self.sprites[2]:setTexture ( path .. "resources/sprites/".. n .."_attack.png", i )
	self.sprites[3]:setTexture ( path .. "resources/sprites/".. n .."_cast.png", i )
	self.sprites[4]:setTexture ( path .. "resources/sprites/".. n .."_hurt.png", i )

	self:setAnimation( ANIMATION_TYPE_IDLE )
end

function  mob:setAniFace()
	if cam_facing == FACING_TYPE_NORTH then
		if self.mob_face == FACING_TYPE_NORTH then
			self.ani_face = 1
		elseif self.mob_face == FACING_TYPE_WEST then
			self.ani_face = 2
		elseif self.mob_face == FACING_TYPE_SOUTH  then
			self.ani_face = 3
		elseif self.mob_face == FACING_TYPE_EAST  then
			self.ani_face = 4
		end
	elseif cam_facing == FACING_TYPE_WEST then
		if self.mob_face == FACING_TYPE_NORTH then
			self.ani_face = 4
		elseif self.mob_face == FACING_TYPE_WEST then
			self.ani_face = 1
		elseif self.mob_face == FACING_TYPE_SOUTH  then
			self.ani_face = 2
		elseif self.mob_face == FACING_TYPE_EAST  then
			self.ani_face = 3
		end
	elseif cam_facing == FACING_TYPE_SOUTH  then
		if self.mob_face == FACING_TYPE_NORTH then
			self.ani_face = 3
		elseif self.mob_face == FACING_TYPE_WEST then
			self.ani_face = 4
		elseif self.mob_face == FACING_TYPE_SOUTH  then
			self.ani_face = 1
		elseif self.mob_face == FACING_TYPE_EAST  then
			self.ani_face = 2
		end
	elseif cam_facing == FACING_TYPE_EAST  then
		if self.mob_face == FACING_TYPE_NORTH then
			self.ani_face = 2
		elseif self.mob_face == FACING_TYPE_WEST then
			self.ani_face = 3
		elseif self.mob_face == FACING_TYPE_SOUTH  then
			self.ani_face = 4
		elseif self.mob_face == FACING_TYPE_EAST  then
			self.ani_face = 1
		end
	end
end

function mob:takeDamage( damage )
	-- print damage effect
	p = self.position
	n = table.getn( effects ) + 1
	effects[ n ] = visual:new()
	effects[ n ]:setPosition( xcore.x3dpoint( p.X, p.Y+5, p.Z ) )
	effects[ n ]:setType( VISTAL_TYPE_TEXT )
	effects[ n ]:setColour( 255, 255, 0, 0 )
	effects[ n ]:setText( "-" ..damage )

	self.hp = self.hp - damage
	if self.hp <= 0 then
		self:setState( STATE_DOWN )
		game_history:addItem ( self:getName() .. " has been defeated." )
		self:setAnimation( ANIMATION_TYPE_DIE )
		if self.owner_id ~= 1 then
			if math.random(1, 100) >= map_droprate then game_map:dropLoot( self.grid.X, self.grid.Y ) end
		end
		return true
	else
		game_history:addItem ( self:getName() .. " has " .. self.hp ..  "HP remaining." )
	end
	return false
end

function mob:updateStatus()
	-- print("[mob] updateStatus")
	if table.getn( self.status_effects ) > 0 then
		for n = 1, table.getn( self.status_effects ) do
			if self.status_effects[ n ].effect_type == EFFECT_TYPE_DAMAGE then
				damage = rollDice( self.status_effects[ n ].effect_damage )
				game_history:addItem ( self.name .. " takes " .. damage .. " points of damage from effect " ..  self.status_effects[ n ].effect_name )
				self:takeDamage( damage )
			end
			if self.status_effects[ n ].effect_duration > 0 then
				self.status_effects[ n ].effect_duration = self.status_effects[ n ].effect_duration - 1
			end
		end
	end
	
	-- remove any status effects with a duration of exactly 0
	n = table.getn( self.status_effects )
	for i = n, 1, -1 do
		if self.status_effects[ i ].effect_duration == 0 then
			print( self.status_effects[ i ].effect_name .. " has expired" )
			self.status_effects[ i ] = nil

			table.remove ( self.status_effects, i )
		end    
	end
end

function mob:canMove()
	for n = 1, table.getn( self.status_effects ) do
		if self.status_effects[ n ].effect_type == EFFECT_TYPE_NO_MOVE or self.status_effects[ n ].effect_type == EFFECT_TYPE_NO_ACT then
			return false
		end
	end
	return true
end

function mob:canAttack()
	for n = 1, table.getn( self.status_effects ) do
		if self.status_effects[ n ].effect_type == EFFECT_TYPE_NO_ATTACK or self.status_effects[ n ].effect_type == EFFECT_TYPE_NO_ACT then
			return false
		end
	end
	return true
end

function mob:canSpecial()
	for n = 1, table.getn( self.status_effects ) do
		if self.status_effects[ n ].effect_type == EFFECT_TYPE_NO_SPECIAL or self.status_effects[ n ].effect_type == EFFECT_TYPE_NO_ACT then
			return false
		end
	end
	return true
end

function mob:updateAnimation()
	self.ani_time = self.ani_time + 1
	if self.ani_time > self.ani_freq then
		self.ani_time = 1
		self.ani_frame = self.ani_frame + 1
		if self.ani_frame >= self.ani_len then
			if self.loop == 1 then
				self.ani_frame = 1
			elseif self.loop == -1 then
				self.ani_frame = 1
				if self.state == STATE_DOWN then
					self:setAnimation( ANIMATION_TYPE_DIE )
				else
					self:setAnimation( ANIMATION_TYPE_IDLE )
				end
			else
				self.ani_frame = self.ani_len
				if self.ani_set == ANIMATION_TYPE_DIE then
					self.wait = self.wait + 1
					if self.wait > 100 then
						-- print("removing corpse")
						self:setGrid( -10000, -10000 )
						self:setAnimation( ANIMATION_TYPE_IDLE )
						self.wait = 0
					end
				end
			end
		end
	end

	if self.ani_set == ANIMATION_TYPE_IDLE then
		-- print("self.type:" .. self.type )
		if tonumber(self.type) == 2 then
			-- print("ani_frame:" ..  self.ani_frame )
			-- print("ani_set:" ..  self.ani_set )
			self.ani_freq   = 50
			self.loop = 1
			self.ani_len = 9
			m:buildTextureTransform( 0, xcore.x2dpoint(0,0), ani_long[ self.ani_frame ][ self.ani_face ], xcore.x2dpoint(1/self.ani_len, 1/4) )
			self.sprites[ ANIMATION_TYPE_WALK ]:setTextureMatrix( m )
		else
			self.ani_frame = 1
			m:buildTextureTransform( 0, xcore.x2dpoint(0,0), ani_long[ self.ani_frame ][ self.ani_face ], xcore.x2dpoint(1/self.ani_len, 1/4) )
			self.sprites[ self.ani_set+1 ]:setTextureMatrix( m )
		end
	elseif self.ani_set == ANIMATION_TYPE_WALK then
		m:buildTextureTransform( 0, xcore.x2dpoint(0,0), ani_long[ self.ani_frame ][ self.ani_face ], xcore.x2dpoint(1/self.ani_len, 1/4) )
		self.sprites[ self.ani_set ]:setTextureMatrix( m )
	elseif self.ani_set == ANIMATION_TYPE_ATTACK then
		m:buildTextureTransform( 0, xcore.x2dpoint(0,0), ani_short[ self.ani_frame ][ self.ani_face ], xcore.x2dpoint(1/self.ani_len, 1/4) )
		self.sprites[ self.ani_set ]:setTextureMatrix( m )
	elseif self.ani_set == ANIMATION_TYPE_SPECIAL then
		m:buildTextureTransform( 0, xcore.x2dpoint(0,0), ani_short[ self.ani_frame ][ self.ani_face ], xcore.x2dpoint(1/self.ani_len, 1/4) )
		self.sprites[ self.ani_set ]:setTextureMatrix( m )
	elseif self.ani_set == ANIMATION_TYPE_DIE then
		m:buildTextureTransform( 0, xcore.x2dpoint(0,0), ani_short[ self.ani_frame ][ self.ani_face ], xcore.x2dpoint(1/self.ani_len, 1/4) )
		self.sprites[ self.ani_set ]:setTextureMatrix( m )
	end
end

function mob:setFacing( d ) -- set direction
	self.mob_face = d
	-- self.ani_time = 1
	-- self.ani_frame = 1
end

function mob:setAnimation( t ) -- set animation type
	if self.ani_set ~= t then
		self.ani_set = t
	end
	self.ani_time = 1
	self.ani_frame = 1
	if t == ANIMATION_TYPE_IDLE then
		self.sprites[ ANIMATION_TYPE_WALK ]:setVisible( true )
		self.sprites[ ANIMATION_TYPE_ATTACK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_SPECIAL ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_DIE ]:setVisible( false )
		self.loop = 0
		self.ani_len = 9
	elseif t == ANIMATION_TYPE_WALK then
		self.sprites[ ANIMATION_TYPE_WALK ]:setVisible( true )
		self.sprites[ ANIMATION_TYPE_ATTACK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_SPECIAL ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_DIE ]:setVisible( false )
		self.loop = 1
		self.ani_len = 9
	elseif t == ANIMATION_TYPE_ATTACK then
		self.sprites[ ANIMATION_TYPE_WALK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_ATTACK ]:setVisible( true )
		self.sprites[ ANIMATION_TYPE_SPECIAL ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_DIE ]:setVisible( false )
		self.loop = -1
		self.ani_len = 6
	elseif t == ANIMATION_TYPE_SPECIAL then
		self.sprites[ ANIMATION_TYPE_WALK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_ATTACK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_SPECIAL ]:setVisible( true )
		self.sprites[ ANIMATION_TYPE_DIE ]:setVisible( false )
		self.loop = -1
		self.ani_len = 6
	elseif t == ANIMATION_TYPE_DIE then
		self.sprites[ ANIMATION_TYPE_WALK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_ATTACK ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_SPECIAL ]:setVisible( false )
		self.sprites[ ANIMATION_TYPE_DIE ]:setVisible( true )
		self.loop = 0
		self.ani_len = 6
	end
end

function mob:targetClosestHostile()
	-- print ("[mob] targetClosestHostile")
	self.range_to_target = 999
	self.target_id = -1
	for i = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ i ].owner_id ~= self.owner_id and game_map.mobs[ i ].state ~= STATE_DOWN then
			-- hostile mob
			range = getDistance( self.grid, game_map.mobs[ i ].grid )
			-- print("[mob] target " .. game_map.mobs[ i ].name .. " at range " .. range)			
			if range > 0 and range < self.range_to_target then
				-- print("[mob] aquired")
				self.range_to_target = range
				self.target_id = game_map.mobs[ i ].mob_id
			end
		end
	end
end


function mob:targetClosestAlly()
	-- print ("[mob] targetClosestAlly")
	self.range_to_target = 999
	self.target_id = -1
	for i = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ i ].owner_id == self.owner_id and game_map.mobs[ i ].state ~= STATE_DOWN then
			-- hostile mob
			range = getDistance( self.grid, game_map.mobs[ i ].grid )
			-- print("[mob] target " .. game_map.mobs[ i ].name .. " at range " .. range)			
			if range > 0 and range < self.range_to_target then
				-- print("[mob] aquired")
				self.range_to_target = range
				self.target_id = game_map.mobs[ i ].mob_id
			end
		end
	end
end

function mob:getAdjacentHostileCount()
	-- print ("[mob] getAdjacentHostileCount")
	local ct = 0
	for i = 1, table.getn( game_map.mobs ) do
		if game_map.mobs[ i ].owner_id ~= self.owner_id and game_map.mobs[ i ].state ~= STATE_DOWN then
			-- active hostile mob
			range = getDistance( self.grid, game_map.mobs[ i ].grid )
			if range == 1 then
				-- print("[mob] adjacent enemy")
				ct = ct+ 1
			end
		end
	end
	return ct
end

function mob:getRandomSpecial()
	-- print("[mob] getRandomSpecial ")
	if table.getn( self.special_abilities ) > 0 then
		local i = math.random(1,  table.getn( self.special_abilities ) )
		return i
	end
	return nil
end

function mob:equipWeapon( n )
	-- print ("[mob] equipWeapon")
	-- print("weapon id " .. n )
	self.wpn = item:new()
	self.wpn:loadItem( n )
	self.wpn_equipped_id = n
	self:setTexture( self.wpn.item_graphic, 3 )
end

function mob:equipArmor( n )
	-- print ("[mob] equipArmor")
	-- print("armor id " .. n )
	self.arm = item:new()
	self.arm:loadItem( n )
	self.arm_equipped_id = n
	self:setTexture( self.arm.item_graphic, 2 )	
end

function mob:equipAccessory( n )
	-- print ("[mob] equipAccessory")
	self.acc = item:new()
	self.acc:loadItem( n )
	self.acc_equipped_id = n
end

function mob:addExp( n )
	self.exp = self.exp + n
end

function mob:setLevel( n )
	if n == nil then return nil end
	if n >= 10  then
		self.exp = 5110
	elseif n == 9 then
		self.exp = 2550
	elseif n == 8 then
		self.exp = 1270
	elseif n == 7 then
		self.exp = 630
	elseif n == 6 then
		self.exp = 310
	elseif n == 5 then
		self.exp = 150
	elseif n == 4 then
		self.exp = 70
	elseif n == 3 then
		self.exp = 30
	elseif n == 2 then
		self.exp = 10
	else
		self.exp = 0
	end
	
end

function mob:getLevel()
	-- print("[mob] getLevel")
	if self.exp >= 5110 then
		return 10
	elseif self.exp >= 2550 then
		return 9
	elseif self.exp >= 1270 then
		return 8
	elseif self.exp >= 630 then
		return 7
	elseif self.exp >= 310 then
		return 6
	elseif self.exp >=150 then
		return 5
	elseif self.exp >= 70 then
		return 4
	elseif self.exp >= 30 then
		return 3
	elseif self.exp >= 10 then
		return 2
	else
		return 1
	end
end

function mob:getEXPToLevel( n )
	if n >= 10  then
		return 5110
	elseif n == 9 then
		return 2550
	elseif n == 8 then
		return 1270
	elseif n == 7 then
		return 630
	elseif n == 6 then
		return 310
	elseif n == 5 then
		return 150
	elseif n == 4 then
		return 70
	elseif n == 3 then
		return 30
	elseif n == 2 then
		return 10
	else
		return 0
	end
	
end

function getRelDir( d )
	if d == FACING_TYPE_NORTH then
		return "FORWARD"
	elseif d == FACING_TYPE_WEST then
		return "LEFT"
	elseif d == FACING_TYPE_SOUTH  then
		return "BACK"
	elseif d == FACING_TYPE_EAST  then
		return "RIGHT"
	end
	return "error"
end

function getDir( d )
	if d == FACING_TYPE_NORTH then
		return "NORTH"
	elseif d == FACING_TYPE_WEST then
		return "WEST"
	elseif d == FACING_TYPE_SOUTH  then
		return "SOUTH"
	elseif d == FACING_TYPE_EAST  then
		return "EAST"
	end
	return "error"
end

-- end of file