-- Dungeon Tactics
-- Copyright 2012 lukas w. irwin [ lukas.w.irwin@gmail.com ]
-- This program is free software under the terms of the GNU General Public License version 3 or later.
-- path.lua

openList = {}
closedList = {}
pathList = {}

function getDistance( pointA, pointB )
	-- print "getDistance"
	xDist = math.abs ( pointA.X - pointB.X )
	yDist = math.abs ( pointA.Y - pointB.Y )
	return yDist + xDist
end

function getTileIDByGridCoords( p )
	-- print "getTileIDByGridCoords"
	-- print ("coords :" .. p.X .. "," .. p.Y )
	-- print ("id" .. game_map.tiles[p.X][p.Y].tile_id )
	return game_map.tiles[ p.X ][ p.Y ].tile_id
end

function getCurrentNode()
	-- print "getCurrentNode"
	curNode = -1
	cost = 999
	openList_size = table.getn( openList )
	-- print ( "openList_size:" .. openList_size )
	
	for i = 1, openList_size do
		-- print ( "Node " .. i )
		if openList[ i ]:getF() < cost then
			-- print "getCurrentNode b"
			cost = openList[ i ]:getF()
			curNode = i
			-- print ( "openList[" .. i .. "].F = " .. cost )
		end
	end
	-- print ( "return " .. curNode )
	return curNode
end

function getParentCost( par )
	-- print "getParentCost"
	closedList_size = table.getn( closedList )
	-- print ( "openList_size:" .. closedList_size )
	for i = 1, closedList_size do
		if closedList[ i ]:getTileID() == par then
			return closedList[ i ]:getG()
		end
	end
	return -1
end

function isOnOpenList( tile )
	-- print "isOnOpenList"
	-- test to see if tile is already on open list
	openList_size = table.getn( openList )
	-- print ( "openList_size:" .. openList_size )
	for i = 1, openList_size do
		-- print ("i:" .. i)
		if openList[ i ]:getTileID() == tile then
			-- print "return true"
			return true
		end
	end
	-- print "return false"
	return false
end

function isOnClosedList( tile )
	-- print "isOnClosedList"
	closedList_size = table.getn( closedList )
	-- print ( "closedList_size:" .. closedList_size )
	-- test to see if tile is already on closed list
	for i = 1, closedList_size do
		-- print ("i:" .. i)
		if closedList[ i ]:getTileID() == tile then
			-- print "return true"
			return true
		end
	end
	-- print "return false"
	return false
end

function getTargetNode( target )
	-- print "getTargetNode"
	closedList_size = table.getn( closedList )
	-- print ( "closedList_size:" .. closedList_size )
	for i = 1, closedList_size do
		-- print ("i:" .. i)
		if closedList[ i ]:getTileID() == target then
			-- print ( "return " .. i )
			return i
		end
	end
	-- print "return -1"
	return -1
end

function getTilePosByID( id )
	-- print "getTilePosByID"
	-- print ("id" .. id )
	for x = 1, map_size do
		for y = 1, map_size do
			if game_map.tiles[ x ][ y ].tile_id == id then
				return xcore.x2dpoint( x, y)
			end
		end
	end
	-- print "[path] error"
	return nil
end

function testTile( x, y, par, dest ) -- int x, int y, int par, wxPoint dest
	-- print "\ntestTile"
	-- print ( "x:" .. x )
	-- print ( "y:" .. y )

	tile_id = game_map.tiles[x][y].tile_id
	-- print ( "tile_id:" .. tile_id )
	-- print ( "par:" .. par )

	-- If it is not walkable or if it is on the closed list, ignore it.
	if tonumber( game_map.tiles[x][y].tile_type ) ~= tonumber( 4 ) and isOnClosedList( tile_id ) == false then

		path_blocked_by_character = 0
		mobAtGrid = getMobIndexAtGrid( xcore.x2dpoint(x, y) )
		if mobAtGrid ~= nil then
			-- print ("characterAtGrid :" .. mobAtGrid )
			-- print ("target_index :" .. target_index )
			if mobAtGrid ~= target_index then
				-- print("path blocked")
				path_blocked_by_character = 1
			end
		else
			-- print ("no character at grid" )
		end

		if path_blocked_by_character == 0 then
			if isOnOpenList( tile_id ) == false then --  If not on the open list, add it.
				-- print "\nAdd new square to open list"
				--   print  ( "pawn :" << getPawnAtGrid( xcore.x3dpoint( x, y ) ) )
				--   print  ( "activePawn :" .. getActivePawn() )
				i = table.getn( openList ) +1
				openList[i] = pathnode:new()
				openList[i]:setTileID( tile_id )
				openList[i]:setParent( par )
				openList[i]:setH( getDistance( xcore.x2dpoint(x, y) , dest) )
				openList[i]:setG( 1+getParentCost( par ) )
				openList[i]:setF( openList[i]:getH() + openList[i]:getG() )
	
					-- print ( "nodetile=" .. openList[i]:getTileID() )
					-- print ( "nodeparent=" .. openList[i]:getParent() )
					-- print ( "H=" .. openList[i]:getH() )
					-- print ( "G=" .. openList[i]:getG() )
					-- print ( "F=" .. openList[i]:getF() )
			else
				-- print "tile already on open list"
			end
		end
	else
		-- print "ignoring tile"
	end
end

function clearLists()
	-- print "\n[path] clearLists"

	for i = 1, table.getn( openList ) do openList[i] = nil end
	for i = 1, table.getn( closedList ) do closedList[i] = nil end
	for i = 1, table.getn( pathList ) do pathList[i] = nil end

	openList = nil
	closedList = nil
	pathList = nil

	openList = {}
	closedList = {}
	pathList = {}
	
end

function findPath( pointA, pointB ) -- return path's length?
	-- print "\n[path] findPath"
	-- print ( "[path] start : " .. pointA.X .. ", " .. pointA.Y )
	-- print ( "[path] end : " .. pointB.X .. ", " .. pointB.Y )

	if pointA.X ==  pointB.X and  pointA.Y ==  pointB.Y then return 0 end 
	
	clearLists()
	
	foundPath = 0
	-- print "add tile at grid pointA to open list."

	openList[1] = pathnode:new()
	openList[1]:setTileID( getTileIDByGridCoords( pointA ) )
	openList[1]:setParent( -1 )
	openList[1]:setH( getDistance( pointA, pointB ) )
	openList[1]:setG( 0 )
	openList[1]:setF( openList[1]:getH() + openList[1]:getG() )

	-- print "[path] Add new square to open list"
	-- print ( "[path] nodetile=" .. openList[1]:getTileID() )
	-- print ( "[path] nodeparent=" .. openList[1]:getParent() )
	-- print ( "[path] H=" .. openList[1]:getH() )
	-- print ( "[path] G=" .. openList[1]:getG() )
	-- print ( "[path] F=" .. openList[1]:getF() )

	startTileID = getTileIDByGridCoords( pointA )
	destTileID = getTileIDByGridCoords( pointB )
	steps = 0
	while foundPath == 0 do
		-- a) Look for the lowest F cost square on the open list. We refer to this as the current square.
		-- print "\nA - Look for the lowest F cost square on the open list. We refer to this as the current square."
		-- print ("step #" ..steps )
		curNode = getCurrentNode()
		-- print ( "\ncurNode:" .. curNode )
		if curNode ~= -1 then
			curTile = openList[ curNode ]:getTileID()
			curGrid = getTilePosByID( curTile )

			-- print ( "closedList_size:" .. table.getn( closedList ) )
			-- print ( "openList_size:" .. table.getn( openList ) )

			-- b) Switch it to the closed list.
			-- print "\nB Switch it to the closed list."
			closedList[ table.getn( closedList )+1 ] =  openList[ curNode ]
			-- openList[ curNode ] = nil
			table.remove(openList, curNode)

			-- print ( "closedList_size:" .. table.getn( closedList ) )
			-- print ( "openList_size:" .. table.getn( openList ) )

			-- c) For each of the 4 squares adjacent to the current square...
			-- print "\nC text each of the 4 squares adjacent to the current square"
			-- print ( "current x : " .. curGrid.X )
			-- print ( "current y : " .. curGrid.Y )

			if curGrid.X-1 > 0 then testTile( curGrid.X-1, curGrid.Y, curTile, pointB ) end
			if curGrid.X+1 < map_size+1 then testTile( curGrid.X+1, curGrid.Y, curTile, pointB ) end
			if curGrid.Y-1 > 0 then testTile( curGrid.X, curGrid.Y-1, curTile, pointB ) end
			if curGrid.Y+1 < map_size+1 then testTile( curGrid.X, curGrid.Y+1, curTile, pointB ) end

			-- diag movement
			-- if curGrid.X-1 > 0 and curGrid.Y-1 > 0 then testTile( curGrid.X-1, curGrid.Y-1, curTile, pointB ) end
			-- if curGrid.X+1 < map_size and curGrid.Y+1 < map_size then testTile( curGrid.X+1, curGrid.Y+1, curTile, pointB ) end
			-- if curGrid.X-1 > 0 and curGrid.Y+1 < map_size then testTile( curGrid.X-1, curGrid.Y+1, curTile, pointB ) end
			-- if curGrid.X+1 < map_size and curGrid.Y-1 > 0 then testTile( curGrid.X+1, curGrid.Y-1, curTile, pointB ) end

			-- d) Stop when
			-- print "\nD test to stop."
			if isOnClosedList( destTileID ) == true then
				foundPath = 1
				-- print ( "[path] found path in " .. steps .. " steps" )
				-- Working backwards from the target square
				-- go from each square to its parent square until you reach the starting square.
				foundStart = 0
				moves = 0

				tgtNode = getTargetNode( destTileID )
				parent  = closedList[ tgtNode ]:getParent()
				pathList[ table.getn( pathList )+1 ] = closedList[ tgtNode ]
				table.remove( closedList, tgtNode )
				while foundStart ~= 1 do
					moves = moves + 1
					tgtNode = getTargetNode( parent )
					if parent ~= -1 then
						-- print("parent:" .. parent )
						-- print("tgtNode:" .. tgtNode )
						parent  = closedList[ tgtNode ]:getParent()
						pathList[ table.getn( pathList )+1 ] = closedList[ tgtNode ]
						table.remove(closedList, tgtNode)

						pathSize = table.getn( pathList )

						if pathList[ pathSize ]:getTileID() == startTileID then
							-- print ( "[path] shortest path is " .. moves .. " moves long" )
							-- print ( "[path] pathSize: " .. pathSize )
							-- game_map.tiles[ pointA.X ][ pointA.Y ].label:setText( txt )

							foundStart = 1
							-- print ( "[path] start : " .. pointA.X .. ", " .. pointA.Y )
							-- print ( "[path] end : " .. pointB.X .. ", " .. pointB.Y )
							-- TODO: fix annotations
							-- game_map.tiles[ pointA.X ][ pointA.Y ].label:setText( "start" )
							-- game_map.tiles[ pointB.X ][ pointB.Y ].label:setText( "end" )

							-- print ( "[path] returning " .. moves )
							return moves
						end
					end
				end
			else
				-- print ( "openList size :" .. table.getn( openList ) )
				if table.getn( openList ) == 0 then
					-- print ( "can not find path in " .. steps .. " steps" )
					foundPath = 1
				end
			end
			-- print "not found; still open; add step"
			steps = steps + 1
			if steps > 256 then
				-- print "taking too long, force exit"
				return -1
			end
		else
			-- print "found find path"
			foundPath = 1
		end
	end
	-- print "can not find path"
	return -1
end

function checkforlosblock(x,y)
	if x < 1 or x > map_size then return false end
	if y < 1 or y > map_size then return false end
	if game_map.tiles[x][y].tile_type == TILE_TYPE_BLOCK then return true end
	return false
end

function lineofsight(x1,y1, x2,y2)
	-- print("[path] lineofsight")
	local deltax = math.abs(x2 - x1) -- // The difference between the x's
	local deltay = math.abs(y2 - y1) -- // The difference between the y's
	local x = x1; -- // Start x off at the first pixel
	local y = y1; -- // Start y off at the first pixel
	local xinc1,xinc2,yinc1,yinc2
	if (x2 >= x1) then -- // The x-values are increasing
		xinc1 = 1
		xinc2 = 1
	else -- // The x-values are decreasing
		xinc1 = -1
		xinc2 = -1
	end

	if (y2 >= y1) then -- // The y-values are increasing
		yinc1 = 1
		yinc2 = 1
	else -- // The y-values are decreasing
		yinc1 = -1
		yinc2 = -1
	end

	if (deltax >= deltay) then -- // There is at least one x-value for every y-value
		xinc1 = 0 -- // Don't change the x when numerator >= denominator
		yinc2 = 0 -- // Don't change the y for every iteration
		den = deltax
		num = deltax / 2
		numadd = deltay
		numpixels = deltax -- // There are more x-values than y-values
	else -- // There is at least one y-value for every x-value
		xinc2 = 0 -- // Don't change the x for every iteration
		yinc1 = 0 -- // Don't change the y when numerator >= denominator
		den = deltay
		num = deltay / 2
		numadd = deltax
		numpixels = deltay -- // There are more y-values than x-values
	end

	local onemoreround = 0
	local roundnum = 0
	while (true) do
		roundnum = roundnum + 1 --forces two distance sight

		 --  // Draw the current pixel
		if onemoreround == 1 and roundnum > 2 then return false end --this stuff is so it penetrates one round and doesnt stop on the first round
		if (roundnum ~= 1) and checkforlosblock(x,y) then onemoreround = 1 end --was return false
		if x == x2 and y == y2 then return true end

		num = num + numadd --// Increase the numerator by the top of the fraction
		if (num >= den) then -- // Check if numerator >= denominator
			num = num - den -- // Calculate the new numerator value
			x = x +xinc1 -- // Change the x as appropriate
			y = y+ yinc1 -- // Change the y as appropriate
		end

		x = x+xinc2 -- // Change the x as appropriate
		y = y+yinc2 -- // Change the y as appropriate
	end
	
	return false
end 

-- Draws a line between two points p1(p1x,p1y) and p2(p2x,p2y).
function getLine( a, b )
	print "getLine"
	print ( "point a : " .. a.X .. "," .. a.Y )
	print ( "point b : " .. b.X .. "," .. b.Y )

	local F
	local x
	local y

	if a.X > b.X then  -- Swap points if p1 is on the right of p2
		print "swap points"
		tmp = a.X
		a.X = b.X
		b.X = tmp
		tmp = a.Y
		a.Y = b.Y
		b.Y = tmp
		print ( "point a : " .. a.X .. "," .. a.Y )
		print ( "point b : " .. b.X .. "," .. b.Y )
	end

	if a.X == b.X then --  Trivial case 1: m = +/-INF (Vertical line)
		print "Trivial case 1"
		if a.Y > b.Y then  -- Swap y-coordinates if p1 is above p2
			print "swap y"
			tmp = a.Y
			a.Y = b.Y
			b.Y = tmp
			print ( "point a : " .. a.X .. "," .. a.Y )
			print ( "point b : " .. b.X .. "," .. b.Y )
		end
	
		x = a.X
		y = a.Y
		while y <= b.Y do
			-- setPixel(x, y) -- TODO: FIX THIS!
			print ("path : " .. x .. ", " .. y)
			y = y + 1
		end
		return 1

	elseif a.Y == b.Y then -- Trivial case 2: m = 0 (Horizontal line)
		print "Trivial case 2"
		x = a.X
		y = a.Y

		while x <= b.X do
			-- setPixel(x, y) -- TODO: FIX THIS!
			print ("path : " .. x .. ", " .. y)
			x = x + 1
		end
		return 1
	end

	local dy            = b.Y - a.Y -- y-increment from p1 to p2
	local dx            = b.X - a.X -- x-increment from p1 to p2
	local dy2           = 2*dy
	local dx2           = 2*dx
	local dy2_minus_dx2 = dy2 - dx2 -- precompute constant for speed up
	local dy2_plus_dx2  = dy2 + dx2


	if dy >= 0 then    --  m >= 0
		-- Case 1: 0 <= m <= 1 (Original case)
		if dy <= dx then
			print "Case 1"
			F = dy2 - dx -- initial F
	
			x = a.X
			y = a.Y
			while x <= b.X do
				-- setPixel(x, y)
				print ("path : " .. x .. ", " .. y)
				if F <= 0 then
					F = F + dy2
				else
					y = y + 1
					F = F + dy2_minus_dx2
				end
				x = x + 1
			end
		-- Case 2: 1 < m < INF (Mirror about y=x line
		-- replace all dy by dx and dx by dy)
		else
			print "Case 2"
			F = dx2 - dy   -- initial F
	
			x = a.X
			y = a.Y
			while y <= b.Y do
				-- setPixel(x, y)
				print ("path : " .. x .. ", " .. y)
				if F <= 0 then
					F = F + dx2
				else
					x=x+1
					F = F - dy2_minus_dx2
				end
				y=y+1
			end
	
		end
	else    -- m < 0
	
		-- Case 3: -1 <= m < 0 (Mirror about x-axis, replace all dy by -dy)
		if dx >= -dy then
			print "Case 3"
			F = -dy2 - dx -- initial F
	
			x = a.X
			y = a.Y
			while x <= b.X do
				print ("path : " .. x .. ", " .. y)	
				-- setPixel(x, y)
				if F <= 0 then
					F = F - dy2
				else
					y = y - 1
					F = F - dy2_plus_dx2
				end
				x = x + 1
			end
		-- Case 4: -INF < m < -1 (Mirror about x-axis and mirror 
		-- about y=x line, replace all dx by -dy and dy by dx)
		else
			print "Case 4"
			F = dx2 + dy -- initial F
	
			x = a.X
			y = a.Y
			while y >= b.Y do
				print ("path : " .. x .. ", " .. y)
				-- setPixel(x, y)
				if F <= 0 then
					F = F + dx2
				else
					x = x + 1
					F = F + dy2_plus_dx2
				end
				y = y - 1
			end
		end
	end
	print "/getLine"
end